#ifndef __CHM_ENGINE_CLASS_H__
#define __CHM_ENGINE_CLASS_H__
//------------------------------------------------------------------------
// Copyright (C) 1997-2007 iNTERFACEWARE Inc.  All Rights Reserved
//
// Module: CHMengineClass
//
// Description:
// Instance of Chameleon engine.
// The CHMengine object represents an instance of a Chameleon
// engine.  Before it can be used the programmer must
// load a message definition file using the Load method of this object.
//
// Author: Eliot Muir
// Date:   Wed 09/09/1998 
// Revision: $Revision: 1.69 $
//
// Last Edit Date: $Date: 2008-07-28 15:33:19 $
// Source: $Source: /home/cvs/cvsroot/CHM/CHMengineClass.h,v $
//---------------------------------------------------------------------------

#include <CHM/CHMminimumInclude.h>

class CHMenginePrivate;
class CHMoutputLog;
class CHMwrapper;
class CHMtransport;
class CHMdateTime;
class CHMstring;
class CHMtable;

/** Instance of Chameleon engine.
 * The CHMengine object represents an instance of a Chameleon
 * engine.  Before it can be used the programmer must
 * load a message definition file using the \c Load method of this object.
 **/
class CHMengine
{
public:
   /** Constructor. */
   CHMengine();
   
   explicit CHMengine(CHMengineHandle handle);

   /** Destructor. */
   virtual ~CHMengine();

    /** Get the index of the ACK message.
    * This message is set in the Options window of Chameleon
    * in the Special Messages tab. 
    * See http://www.interfaceware.com/manual/autoack.html for more information.
    */
   size_t AckMessageIndex() const;   
   
   /** Get the build number of Chameleon. */
   size_t buildNumber() const;
   
   /** Get the name of the configuration specified by 
    *  \c ConfigurationIndex.
    */ 
   const CHMchar* ConfigurationName(size_t ConfigurationIndex) const;
   
   /** Get the number of configurations in the message 
    * definition (VMD) file. 
    */ 
   size_t CountOfConfiguration() const;
   
   /** Get the count of messages. 
    * This method gives the number of message definitions loaded in
    * \c this CHMengine object from the message definition file (VMD).
    * Usage of this function with \c MessageName offers an easy way to iterate 
    * through the messages.
    */
   size_t CountOfMessage() const;

   /** Get the count of tables. 
    * This method gives the number of table definitions loaded in
    * \c this CHMengine object from the message definition file (VMD).
    */
   size_t CountOfTable() const;
   
   /** Get the current configuration index of Chameleon */
   size_t CurrentConfigurationIndex() const;


    /** Get the handle to the underlying Chameleon parser object. 
    * This method is part of the internal implementation of Chameleon and should not be called. 
    */
   const CHMengineHandle EngineHandle() const;

    /** 
    *  Add \c pPath to the path that Python will use to locate your
    *  python modules.  This is in addition to the path
    *  set in the environment variable CHM_PYTHON_LIB_PATH.
    *  A semi-colon (;) delimited string is expected as input in a Windows environment.
    *  A colon (:) delimited string is expected as input in a POSIX environment.
    */
   void extendPythonLibraryPath(const char* pPath);

    /** Fast optimized method to generate an ACK message based on \c pMessage.
    *  Please read the chapter on AutoAcknowledgement in the manual http://www.interfaceware.com/manual/autoack.html
    */
   const CHMchar* generateAck(const CHMchar* pMessage);
   
   /** Generate an XSD to validate XML based on the XML generation style specified in the loaded VMD.  The XSD will
    * be output to \c pFileName.
    */
   void generateXsd( const char* pFilename );
   
   /** Get the index of the default message.
    * This message is used to identify messages which do not match
    * the precise identification criteria for other messages in the
    * given message definition file.  This message is set in the Options window in
    * the Special Messages tab.
    * See http://www.interfaceware.com/manual/default_message.html for more information.
    */
   size_t IgnoreMessageIndex() const;

   /** Get the index of the input configuration used for graphical transformation.
    * The index returned by this method is that of the input configuration last set by the user in 
    * the Transformation window in the VMD file.  This is \b not used for scripted transformation.
    */
   size_t InConfigurationIndex() const;
   
   /** Initiate a message. 
    * \deprecated 
     * This function will attempt to send the message given in the
    * \c pFlatWire argument.  It will use the assigned CHMtransport and
    * CHMwrapper objects that are assigned to this CHMengine object.
    */
   virtual void InitiateMessage(const CHMchar* pFlatWire);
   
   /** 
    *  Check whether the engine is using passthru 
    *  mapping (scripted transformation), or graphical mapping (graphical transformation). 
    */
   CHMboolean IsUsingPassthruMapping() const;
   
   /** Load the visual message definition (VMD) file \c pFileName. 
    * This must be done before the Chameleon engine can be used.
    * Applications should load the VMD file as part of initialization since loading may take a few seconds.
    * Loading the VMD file every time a message is encountered will result in very slow performance.
    */
   virtual void Load(const CHMchar* pFileName);

   
   /** Get a non-const pointer to the CHMoutputLog object assigned to \c this engine. */
   CHMoutputLog* Log();
   
   /** Get the major version number of Chameleon. */
   size_t majorVersion() const;
   
   /** Get the handle to the underlying Message at \c MessageIndex. 
    * This method is part of the internal implementation of Chameleon and should not be called.  
    */
   CHMtableHandle MessageHandle(size_t MessageIndex) const;

   /** Get the message name.
    * This method returns the name of the message located at the
    * index \c MessageIndex. Usage of this function with \c CountOfMessage offers an 
    * easy way to iterate through the messages.
    **/
   const CHMchar* MessageName(size_t MessageIndex) const;

   /** Get the minor version number of Chameleon. */
   size_t minorVersion() const;
   
   /** Get a newline as defined by the current CHMoutputLog object.
    * Because conventions for newlines differ between Windows edit
    * controls and log files, this should be used to ensure that
    * inserting a newline has the desired behavior in a given log
    * output.
    */
   const CHMchar* NewLine() const;

   /** Get the index of the output configuration for graphical transformation.
    * The index returned by this method is that of the output configuration last set by the user in 
    * the Transformation window in the VMD file.  This is \b not used for scripted transformation.
    */
   size_t OutConfigurationIndex() const;

   /** Output \c pLine to the currently assigned CHMoutputLog object.
    */
   void OutputLine(const CHMchar* pLine) const;
   
   /** Parse the message \c pFlatWire from the client at \c TransportIndex.
    * This function needs to be overridden by child classes of CHMengine.
    **/
   virtual void ParseMessage(const CHMchar* pFlatWire, size_t TransportIndex);

   /** Parse the message \c pFlatWire dynamically.
    * This method can be used to parse messages without relying
    * on the stubcode generation of Chameleon.  See 
    * http://www.interfaceware.com/manual/cpp_dynamic_interface.html for more information. 
    * An important
    * thing to note about this interface is that you must create
    * a new CHMtable object each time before calling this method.
    **/
   virtual void ParseMessageDynamically(const CHMchar* pFlatWire, 
                                        CHMtable&   Table, 
                                        size_t&     MessageIndex) const;

   /** Reply to message \c FlatWire from the client at \c TransportIndex.    
    * \deprecated
    * This function uses the assigned CHMtransport and CHMwrapper classes
    * to route a reply message back to the appropriate sender using the
    * \c TransportIndex to look up that sender.
    **/
   virtual void ReplyToMessage(const CHMchar* FlatWire, size_t TransportIndex);

   /** Set the current configuration index of Chameleon to \c Index.
    */
   void SetCurrentConfigurationIndex(size_t Index);
    
   /** 
    *  Programmatically sets the default database associated
    *  with the currently selected configuration at run-time.
    *  @param pAPI is the  database type.  Choose from one of the following strings: MySQL, OCI, ODBC - PostgreSQL,
    *  ODBC - MS Access, ODBC - Oracle, ODBC - MS SQL Server, ODBC - Sybase ASA, ODBC - Sybase ASE.
    *  @param pName is the name of the database.
     * @param pUserName is the user name you are using to access the database.
    *  @param pPassword is the password corresponding to the user name.
    */
   void setDefaultDatabase
   (
      const char* pAPI, 
      const char* pName,
      const char* pUserName,
      const char* pPassword
   );
   
   /** Assign a CHMoutputLog object \c pLog to \c this engine.
    * This method is used by the engine for logging output.  Use of the python "log()" command in the vmd file
    * will be output through the CHMoutputLog interface. Be aware this method takes ownership of the 
    * CHMoutputLog object, i.e., the CHMengine destructor will delete the CHMoutputLog object.
    */
   void SetLog(CHMoutputLog* pLog);
  
   /** Assign a string value to a Python global variable.
    *  Note that the assigned value is always of string type. If an integer or
    *  other type is called for, you will need to convert the value to that
    *  type.*/
   void setPythonKeyValuePair(const char* pKey, const char* pValue);

   /** Set the strict grammar checking flag. 
    * When this flag is set the parser will check for the presence
    * of compulsory fields in each segment and enforce other rules such as maximum field length.  
    * This flag can also be set through the Options screen of Chameleon.
    **/
   void SetRejectBadSegmentGrammarFlag(CHMboolean Flag);
 
   /** Assign a CHMtransport object to \c this engine.
    * \deprecated 
    * The CHMtransport object \c pTransport is used by the engine to send and receive
    * messages.
    * Be aware that this method takes ownership of the assigned CHMtransport
    * object, i.e., the CHMengine destructor will delete the CHMtransport object.
    */
   void SetTransport(CHMtransport* pTransport);
     
   /** Assign a CHMwrapper object to \c this engine.
    * \deprecated 
    * The CHMwrapper object \c pWrapper is used to wrap outgoing messages and unwrap
    * incoming messages from the CHMtransport class.
    * Be aware that this method takes ownership of the assigned CHMwrapper
    * object, i.e., the CHMengine destructor will delete the CHMwrapper object.
    */
   void SetWrapper(CHMwrapper* pWrapper);
   
   /** Get the handle to the underlying Table at \c TableIndex.  
    * This method is part of the internal implementation of Chameleon and should not be called.
    */
   CHMtableHandle TableHandle(size_t TableIndex) const;
   
   /** Transform a message from one format to another.
    * This function invokes Chameleon's translation logic to take
    * a message of the \c InConfigIndex flavor and translate it to the
    * \c OutConfigIndex flavor.    
    * The \c MessageIndex is set according to the type of message that
    * the data \c pMessage was recognized as.  The function returns
    * the transformed data.  If the translation fails, an exception
    * is raised.  See http://www.interfaceware.com/manual/transform_message.html for more information.
    */
   const CHMchar* transformMessage(const CHMchar* pMessage,
                          size_t InConfigIndex,
                          size_t OutConfigIndex,
                          size_t& MessageIndex);

   /** Translate the 2.X HL7 message \c FlatWire into XML using the DTDs of the format specified
    * in the XML tab of the Options window.  See http://www.interfaceware.com/manual/xml.html for more information.
    */
   const char* translateMessageToXml(const char* pFlatWire);

    /** Translate the XML message \c pXml conforming to the DTDs of the format specified in the
    * XML tab of the Options window into a 2.X HL7 message.  See http://www.interfaceware.com/manual/xml.html 
    * for more information.
    */
   const char* translateXmlToMessage(const char* pXml);  
   
   /**  Get a const pointer to the CHMtransport object assigned to \c this engine. 
    * \deprecated */
   const CHMtransport* Transport() const;
   
   /** Get a non-const pointer to the CHMtransport object assigned to \c this engine.
    *  \deprecated */
   CHMtransport* Transport();

   /** Get the version string. */
   const CHMchar* versionString() const;

   /** Get a const pointer to the CHMwrapper object assigned to \c this engine. 
    * \deprecated */
   const CHMwrapper* Wrapper() const;

   /** Get a non-const pointer to the CHMwrapper object assigned to \c this engine. 
    *  \deprecated */
   CHMwrapper* Wrapper();

   
   // This function is deprecated.
   //
   // void OutputStream(IOstrstream& Stream) const;
   
   // This function is deprecated. Use Dump instead. See manual for migration guide.
   //
   // void printOn(IOostream& Stream) const;

   // This function is deprecated. Use Dump instead. See manual for migration guide.
   //
   // void printOn(CHMstring& String) const;

private:
   CHMengine& operator=(const CHMengine& Original); // not allowed
   CHMengine(const CHMengine& Original);            // not allowed
   CHMenginePrivate* pMember;
};

// This function is deprecated. Use Dump instead. See manual for migration guide.
//
// IOostream& operator<<(IOostream& Stream, const CHMengine& Engine);

#endif // end of defensive include
